import { join } from 'node:path';
import process from 'node:process';
import enquirer from 'enquirer';
import chalk from 'chalk';
import React from 'react';
import { useInput, Text, Box } from 'ink';
import { findUp, pathExists } from 'find-up';
import fastGlob from 'fast-glob';
import fs from 'fs-extra';
import { marked } from 'marked';
import { markedTerminal } from 'marked-terminal';
import FlexSearch from 'flexsearch';

// @see https://github.com/mikaelbr/marked-terminal#options
//
marked.use(markedTerminal({
  code: chalk.yellow,
  blockquote: chalk.gray.italic,
  html: chalk.gray,
  heading: chalk.green.bold,
  firstHeading: chalk.magenta.underline.bold,
  hr: chalk.reset,
  listitem: chalk.reset,
  table: chalk.reset,
  paragraph: chalk.reset,
  strong: chalk.bgWhite.black.bold,
  em: chalk.italic,
  codespan: chalk.yellow,
  del: chalk.dim.gray.strikethrough,
  link: chalk.cyan,
  href: chalk.cyan.underline,
  width: 80,
  // Only applicable when reflow is true
  reflowText: true,
  showSectionPrefix: true,
  unescape: true,
  emoji: true,
  tableOptions: {},
  tab: 2
}));
const {
  prompt
} = enquirer;
const {
  Index
} = FlexSearch;
const index = new Index({
  worker: true,
  charset: 'latin:extra',
  tokenize: 'reverse',
  resolution: 9,
  language: 'en',
  filter: [`in`, `into`, `is`, `isn't`, `it`, `it's`]
});
let docList;
await findUp(async directory => {
  const target = join(directory, 'docs');
  if (await pathExists(target)) {
    docList = await fastGlob.glob(join(target, '**/*.md'));
    return target;
  }
}, {
  type: 'directory'
});
if (docList.length === 0) {
  console.log('No /docs folder found. Exiting');
  process.exit(0);
}
const docLoaders = docList.map(docPath => {
  return new Promise((resolve, reject) => {
    fs.readFile(docPath, 'utf8').then(text => {
      resolve(index.add(docPath, text));
    }).catch(reject);
  });
});
await Promise.allSettled(docLoaders);
let promise;
let state;
let selection;
let lastKnownQuery;
const doSearch = () => {
  if (!promise) {
    promise = prompt({
      type: 'autocomplete',
      name: 'fileTarget',
      message: 'Search Docs:',
      limit: 10,
      choices: [],
      suggest(input) {
        lastKnownQuery = input;
        const matches = index.search(input);
        return matches.map(docPath => {
          return {
            value: docPath,
            message: docPath.split('docs')[1]
          };
        });
      }
    });
    state = 'pending';
    promise.then(answer => {
      selection = answer;
      state = 'done';
    });
  }
  if (state === 'pending') {
    throw promise;
  }
  if (state === 'done') {
    return selection;
  }
};
const Search = () => {
  doSearch();
  let rawMarkdown = fs.readFileSync(selection.fileTarget, 'utf8');
  const regex = new RegExp(lastKnownQuery, 'gi');
  rawMarkdown = rawMarkdown.replace(regex, match => {
    return `**${match}**`;
  });
  const markdown = marked.parse(rawMarkdown);
  return /*#__PURE__*/React.createElement(Box, null, /*#__PURE__*/React.createElement(Text, null, markdown));
};
export default Search;